@@ -3,6 +3,8 @@ import string
 from decimal import Decimal
 import csv
 import re
+import unidecode
+from collections import OrderedDict
 from witchcraft.combinators import execute, query, template
 from witchcraft.dateutil.parser import parse as dateutil_parse
 
@@ -35,9 +37,17 @@ def remove_metadata(data_points):
             del item['_created_at']
 
 
-def create_table(connection, schema_name, table_name, fields, primary_keys):
+def create_table(connection, schema_name, table_name, fields, primary_keys, version_column=None):
+
+    primary_keys = list(primary_keys)
+
+
+    columns = list(fields.items())
+
+    if version_column is not None:
+        primary_keys.append(version_column)
+        columns.append((version_column, {'psql_type': 'bigint'}))
 
-    columns = fields.items()
     prefix = prefix_dict.get(connection.database_type)
     execute(connection, template('%s_create_table' % prefix, 
                             dict(columns=columns,
@@ -50,8 +60,11 @@ def create_table(connection, schema_name, table_name, fields, primary_keys):
 # 
 
 def prepare_table(connection, schema_name, table_name, data_points, primary_keys, version_column=None):
+    fields = OrderedDict([])
+
+    for dp in data_points:
+        fields.update(dp.fields)
 
-    fields = data_points[0].fields
     prefix = prefix_dict.get(connection.database_type)
 
     required_columns = find_keys(data_points)
@@ -104,7 +117,7 @@ def prepare_table(connection, schema_name, table_name, data_points, primary_keys
                                 dict(schema_name=schema_name,
                                      table_name=table_name,
                                      column_name=version_column,
-                                     column_type='integer')))
+                                     column_type='bigint')))
 
     return primary_keys
 
@@ -194,7 +207,7 @@ def discover_columns(connection, schema_name, table_name):
     return result
 
 
-def get_max_version(connection, schema_name, table_name):
+def get_max_version(connection, schema_name, table_name, version_column='version'):
     prefix = prefix_dict.get(connection.database_type)
 
     result = query(connection, template('%s_discover_columns' % prefix,
@@ -206,13 +219,14 @@ def get_max_version(connection, schema_name, table_name):
 
     tpl = template('%s_max_version' % prefix,
                      dict(schema_name=schema_name,
-                          table_name=table_name),
+                          table_name=table_name,
+                          version_column=version_column),
                      connection.database_type)
 
     result = query(connection, tpl)
 
-    if len(result) > 0 and result[0].version is not None:
-        return result[0].version+1
+    if len(result) > 0 and getattr(result[0], version_column) is not None:
+        return getattr(result[0], version_column) + 1
     else:
         return 1
 
@@ -288,13 +302,30 @@ def extract_number(number_str, decimal=None):
         else:
             return value, InputType('text')
 
-    elif len(groups) > 1:
+    # xx.xx
+    elif len(groups) == 2:
         number = Decimal('%s%s.%s' % (sign,''.join(groups[0:-1]), groups[-1]))
         precision = len(number.as_tuple().digits)
         scale = - number.as_tuple().exponent
         input_type = InputType('numeric', dict(precision=precision, scale=scale))
         return number, input_type
 
+    # x.xxx,xx / x,xxx.xx
+    elif len(groups) == 3 and separators[-1] != separators[-2]:
+        number = Decimal('%s%s.%s' % (sign,''.join(groups[0:-1]), groups[-1]))
+        precision = len(number.as_tuple().digits)
+        scale = - number.as_tuple().exponent
+        input_type = InputType('numeric', dict(precision=precision, scale=scale))
+        return number, input_type
+
+    elif len(groups) > 3 and separators[-1] != separators[-2]:
+        number = Decimal('%s%s.%s' % (sign,''.join(groups[0:-1]), groups[-1]))
+        precision = len(number.as_tuple().digits)
+        scale = - number.as_tuple().exponent
+        input_type = InputType('numeric', dict(precision=precision, scale=scale))
+        return number, input_type
+
+
     else:
         return None
 
@@ -330,7 +361,7 @@ def parse_csv(input_data, delimiter=';', quotechar='"'):
 
     sniffer = csv.Sniffer()
     try:
-        dialect = sniffer.sniff(input_data, delimiters=';\t')
+        dialect = sniffer.sniff(input_data, delimiters=';\t,')
     except:
         csv.register_dialect('dlb_excel', delimiter=delimiter, quotechar=quotechar)
         dialect = csv.get_dialect('dlb_excel')
@@ -405,6 +436,9 @@ def detect_type(value, current_type=None):
             precision = max(current_type.params['precision'], natural_lenght)
             scale = current_type.params['scale']
             precision = max(natural_lenght + scale, precision)
+
+        else:
+            return value, InputType('text')
             
         return value, InputType('numeric', dict(precision=precision, scale=scale))
 
@@ -525,6 +559,12 @@ def get_data_types(header, data, current_types=None, detect_type_func=None):
 
         result_data.append(result_row)
 
+
+    for h in header:
+        if h not in current_types:
+            current_types[h] = 'text'
+        
+
     return result_data, current_types
 
 
@@ -535,7 +575,7 @@ def format_header(header):
     generic_name_counter = 0
 
     for column in header:
-
+        column = unidecode.unidecode(column)
         letters_and_digits = string.ascii_lowercase + string.digits
         buf = ''
 
